package com.supermap.plotting;

/**
 * <p>
 * Title:Situation Plotting
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Statement----------------------------
 * This is SuperMap iMobile for Android Sample Code 
 * Copyright: SuperMap Software Co., Ltd.
 * ----------------------------------------------------------------
 * ---------------------SuperMap iMobile for Android Sample Code------------------------
 * 
 * 1. Introduction:
 *   	The usage of situation plotting
 *   
 * 2. Data:
 * 		Directory: "../SampleData/PlottingData/"
 *      Map data: "SuperMapCloud.smwu", "Edit.udb", "Edit.udd"
 *      License: "../SuperMap/License/"
 *      
 * 3. Key Member
 *    addPlotLibrary();				Method
 *    setPlotSymbol();				Method
 *    setAction();					Method
 *    getCurrentGeometry();			Method
 *    deleteCurrentGeometry();		Method
 *    submit();						Method
 *    undo();						Method		
 *    redo();						Method
 *    cancel();						Method
 *
 * 4. Function
 *   (1)Draw marker plotting symbol
 *   (2)Draw arrow plotting symbol
 *   (3)Edit and delete plotting symbol
 *   (4)Undo, redo
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.GridView;
import android.widget.SimpleAdapter;
import android.widget.AdapterView;
import android.widget.Toast;

import com.supermap.data.*;
import com.supermap.mapping.*;


public class MainActivity extends Activity {

	private MapControl m_mapControl = null;
	private Workspace m_workSpace = null;
	private MapView m_mapView = null;

	private GridView m_gridViewPoint = null;	//GridView of marker plotting symbol
	private GridView m_gridViewLine = null;		//GridView of line plotting symbol
	private List<Map<String, Object>> m_dataListPoint = null;	//Resource list of marker plotting symbol
	private List<Map<String, Object>> m_dataListLine = null;	//Resource list of line plotting symbol
	//private SimpleAdapter m_simpleAdaper = null;
	
	private long libID_JB = -1;	//ID of marker symbol library
	private long libID_TY = -1;	//ID of line symbol library
	
	//Image resource for marker symbol.This function needs to add the resource file to project to display the marker symbol in GridView
    private int[] iconPoint = { R.drawable.point_10, R.drawable.point_10100,
			R.drawable.point_14, R.drawable.point_30304, R.drawable.point_30307,
			R.drawable.point_30308, R.drawable.point_30502, R.drawable.point_30709,
			R.drawable.point_40503, R.drawable.point_56, R.drawable.point_70100,
			R.drawable.point_80102 , R.drawable.point_80106, R.drawable.point_80201,
			R.drawable.point_80400};
	// The name/ID of marker symbol
    private String[] iconNamePoint = { "10", "10100", "14", "30304", "30307", "30308", "30502",
			"30709", "40503", "56", "70100", "80102", "80106", "80201", "80400" };
	//The image resource for line symbol. This function needs to add the resource file to project to display the line symbol in GridView
	private int[] iconLine = { R.drawable.line_1000, R.drawable.line_1001,
				R.drawable.line_1002, R.drawable.line_1003, R.drawable.line_1004,
				R.drawable.line_1005, R.drawable.line_1006, R.drawable.line_1007,
				R.drawable.line_1008, R.drawable.line_1009 };
	//The name/ID of line symbol
	private String[] iconNameLine = { "1000", "1001", "1002", "1003", "1004", "1005", "1006",
				"1007", "1008", "1009" };
	
	@Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        String sdcard = android.os.Environment.getExternalStorageDirectory().getAbsolutePath().toString();
        String dataPath = sdcard + "/SampleData/PlottingData/";	//Data path. When running this program, the specified data needs to be put under the path
        
        //Set path for license
        Environment.setLicensePath(sdcard + "/SuperMap/license/");
             
        //After initializing Environment, the functions for components can be called.
        Environment.initialization(this);

        setContentView(R.layout.activity_main);
           
      	//Open workspace
  		m_workSpace = new Workspace();
        WorkspaceConnectionInfo info = new WorkspaceConnectionInfo(); 
        info.setServer(dataPath + "SuperMapCloud.smwu");
        info.setType(WorkspaceType.SMWU);
        boolean result = m_workSpace.open(info);  
        if (!result) {
			
        	Toast.makeText(this, "Failed to open workspace!", Toast.LENGTH_LONG).show();
			
			m_workSpace.close();
			m_workSpace = null;
			
			return;
		}
        
        //Associate the map display control with the workspace
        m_mapView = (MapView)findViewById(R.id.Map_view);
        m_mapControl =  m_mapView.getMapControl();
        m_mapControl.getMap().setWorkspace(m_workSpace);
        
        //Open the map in worksapce.
        String mapName = m_workSpace.getMaps().get(0);
        result = m_mapControl.getMap().open(mapName);
        if (!result) {
        	
        	Toast.makeText(this, "Failed to open map!", Toast.LENGTH_LONG).show();
        	
        	m_mapControl.dispose();
        	m_workSpace.close();
        	m_mapControl = null;
        	m_mapView = null;
        	m_workSpace = null;
        	
			return;
		}
        
        //Loading point symbol library
        String libraryJB = dataPath + "Symbol/JB.plot";
        libID_JB = m_mapControl.addPlotLibrary(libraryJB);
        if(libID_JB < 0)
        {
        	Toast.makeText(this, "Failed to load marker symbol library!", Toast.LENGTH_LONG).show();
        	return;
        }
        //Loading line symbol library
        String libraryTY = dataPath + "Symbol/TY.plot";
        libID_TY = m_mapControl.addPlotLibrary(libraryTY);
        if(libID_TY < 0)
        {
        	Toast.makeText(this, "Failed to load line symbol library!", Toast.LENGTH_LONG).show();
        	return;
        }
        
        //Set GridView isn't visible
        m_gridViewPoint = (GridView)findViewById(R.id.gridviewPoint);
        m_gridViewPoint.setVisibility(View.INVISIBLE);
        m_gridViewLine = (GridView)findViewById(R.id.gridviewLine);
        m_gridViewLine.setVisibility(View.INVISIBLE);
        
        //Gets the point symbol data and configs adapter
        m_dataListPoint = new ArrayList<Map<String, Object>>();
      	getPointData();	//Gets data
      	String [] from ={"image","text"};
      	int [] to = {R.id.image,R.id.text};
      	SimpleAdapter simpleAdaperPoint = new SimpleAdapter(this, m_dataListPoint, R.layout.item, from, to);//New adapter
      	m_gridViewPoint.setAdapter(simpleAdaperPoint);//Config adapter
      	
      	//Gets the line symbol data and configs adapter
      	m_dataListLine = new ArrayList<Map<String, Object>>();
       	getLineData();	//Gets data
       	SimpleAdapter simpleAdaperLine = new SimpleAdapter(this, m_dataListLine, R.layout.item, from, to);
       	m_gridViewLine.setAdapter(simpleAdaperLine);//Config adapter
       
       	
       	m_gridViewPoint.setOnItemClickListener(new OnItemClickListener(){
       		@Override
       		public void onItemClick(AdapterView<?> parent, View view, int position,
       		long id) {

       			String name = iconNamePoint[position];//Gets the symbol name according to the clicking position
       			int symbolCode = Integer.parseInt(name);//Converts the symbol name to an integer type
       			m_mapControl.setPlotSymbol(libID_JB, symbolCode);//Set a situation plotting symbol to be drawn
       			m_mapControl.setAction(Action.CREATEPLOT);//Set the map status to the status of situation plotting
       		}
       	});	
       	
       	m_gridViewLine.setOnItemClickListener(new OnItemClickListener(){
       		@Override
       		public void onItemClick(AdapterView<?> parent, View view, int position,
       		long id) {

       			String name = iconNameLine[position];//Gets the symbol name according to the clicking position
       			int symbolCode = Integer.parseInt(name);//Converts the symbol name to an integer type
       			m_mapControl.setPlotSymbol(libID_TY, symbolCode);//Set a situation plotting symbol to be drawn
       			m_mapControl.setAction(Action.CREATEPLOT);//Set the map status to the status of situation plotting
       		}
       	});	
       	
       	//Set the edit layer of CAD type for Situation plotting
       	Layers layers = m_mapControl.getMap().getLayers();
       	Layer layer = layers.get("CAD@Edit");
        layer.setEditable(true);  
        
		m_mapControl.addGeometrySelectedListener(geoSelectedListener);
		
    }
	
	//The listener of the selected object 
	private GeometrySelectedListener geoSelectedListener = new GeometrySelectedListener() {
		
		@Override
		public void geometrySelected(GeometrySelectedEvent arg0) {
			// TODO Auto-generated method stub
			m_mapControl.setAction(Action.VERTEXEDIT);//The map switches to the status for editing nodes when the object is selected
		}

		@Override
		public void geometryMultiSelected(ArrayList<GeometrySelectedEvent> arg0) {
			// TODO Auto-generated method stub
		}
	};
	
	//Get the point plotting data
	public List<Map<String, Object>> getPointData(){		
		
		for(int i=0;i<iconPoint.length;i++){
			Map<String, Object> map = new HashMap<String, Object>();
			map.put("image", iconPoint[i]);
			map.put("text", iconNamePoint[i]);
			m_dataListPoint.add(map);
		}	
		return m_dataListPoint;
	}
	
	//Get the line plotting data
	public List<Map<String, Object>> getLineData(){		
		
		for(int i=0;i<iconLine.length;i++){
			Map<String, Object> map = new HashMap<String, Object>();
			map.put("image", iconLine[i]);
			map.put("text", iconNameLine[i]);
			m_dataListLine.add(map);
		}
			
		return m_dataListLine;
	}
	
  //Point plotting button
  public void btnPoint_Click(View view){
  	
	  if(m_gridViewPoint.getVisibility() == view.VISIBLE){
		  m_gridViewPoint.setVisibility(view.GONE);
	  }
	  else{
		  m_gridViewPoint.setVisibility(view.VISIBLE);
	  }
	  if(m_gridViewLine.getVisibility() == view.VISIBLE){
		  m_gridViewLine.setVisibility(view.GONE);
	  }
  }
  //Arrow plotting button
  public void btnSymbol_Click(View view){
	  
	  if(m_gridViewLine.getVisibility() == view.VISIBLE){
		  m_gridViewLine.setVisibility(view.GONE);
	  }
	  else{
		  m_gridViewLine.setVisibility(view.VISIBLE);
	  }
	  if(m_gridViewPoint.getVisibility() == view.VISIBLE){
		  m_gridViewPoint.setVisibility(view.GONE);
	  }
  }
  //'Edit symbol' button
  public void btnEdit_Click(View view){
	  
	  m_mapControl.setAction(Action.SELECT);
  }
  //'Delete symbol' button
  public void btnDelete_Click(View view){
	  
	  Geometry geo = m_mapControl.getCurrentGeometry();
		if(geo != null){
			geo.dispose();
			m_mapControl.deleteCurrentGeometry();
			m_mapControl.submit();
			m_mapControl.setAction(Action.PAN);
			m_mapControl.getMap().refresh();
		} else { 

			m_mapControl.setAction(Action.SELECT);
			Toast.makeText(this, "Object isn't selected", Toast.LENGTH_LONG).show();
		}
  }
  //Redo
  public void btnRedo_Click(View view){
  	
	  m_mapControl.redo();
	  m_mapControl.getMap().refresh();
  }
  //Undo
  public void btnUndo_Click(View view){
  	
	  m_mapControl.undo();
	  m_mapControl.getMap().refresh();
  }
  //Cancel
  public void btnCancel_Click(View view){
  	
	  m_mapControl.cancel();
	  m_mapControl.submit();
  }
  //Submit
  public void btnSubmit_Click(View view){
	  
	  Geometry geometry = m_mapControl.getCurrentGeometry();//Gets the object drawing currently
		
	  if(geometry != null){
		  
		  geometry.dispose();	//Release the resource occupied by object
		  
		  Layer layer = m_mapControl.getEditLayer();//Gets the current edit layer
		  if(layer != null){
			  
			  Dataset dataset = layer.getDataset();//Gets the dataset object for the edit layer
			  DatasetType type = dataset.getType();//Gets the dataset type
			  if(type != DatasetType.CAD){//The plotting symbol only can be in CAD layer
				  Toast.makeText(this, "The current layer is not a CAD layer, the symbol can't be submitted!", Toast.LENGTH_LONG).show();
			  } else {
				  m_mapControl.submit();//Submit
				  m_mapControl.setAction(Action.PAN);//Switches the Pan status
			  }
		  }
		  
	  } else {
		  
		  Toast.makeText(this, "No geometric objects are being edited!", Toast.LENGTH_LONG).show();
	  }
  }
  //Empty
  public void btnClear_Click(View view){
  	
	  m_mapControl.cancel();//Cancel
	  
	  Layer layer = m_mapControl.getEditLayer();//Gets the current edit layer
	  DatasetVector datasetVector = (DatasetVector)layer.getDataset();//Gets the dataset object for the edit layer
	  Recordset recordset = datasetVector.getRecordset(false, CursorType.DYNAMIC);
	  if (recordset != null){
		  recordset.deleteAll();
		  recordset.update();
		  recordset.dispose();
		  recordset = null;
	  }
	  m_mapControl.getMap().refresh();
  }
}
